cProgram to illustrate scattering and interface to SeBa
cThis computes scattering of a binary in a multi-mass field of stars with 
cuniform density and velocity dispersion, using the scattering cross section 
cof Giersz (MNRAS, 2001, 324, 218-30) and SeBa to evolve the binary and 
csingle stars.  Exchanges are not yet allowed.
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c Version 1.0: 25 February 2003
c         Douglas C. Heggie           <d.c.heggie@ed.ac.uk>
c         Simon F. Portegies Zwart    <spz@science.uva.nl>
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c Version 2.0: 3 June 2003
c         Douglas C. Heggie           <d.c.heggie@ed.ac.uk>
c         Simon F. Portegies Zwart    <spz@science.uva.nl>
cThis version will use an improved cross section, and 
cincludes exchanges.
c
cVersion 2.0 
c   Cleanup.
c   Exchange probability can be >1
c   Energy change after exchange. Current implementation preserved 
c     semi-major axis.
c   Documentation and explanation
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c Version 3.0: 3 September 2003
c         Douglas C. Heggie           <d.c.heggie@ed.ac.uk>
c         Simon F. Portegies Zwart    <spz@science.uva.nl>
cThis version will be cleaned and performance will be improved
c
cVersion 3.0 
c   Cleanup.
c   Documentation and explanation
c   reshaping the interface
c   writing the research note
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
      program mcscatter
      implicit double precision (a-h,o-z)
      double precision n,m_min,m_max, m1,m2,m3, ma, m3old
      integer outcome, scatter_counter, evolve_counter
cWrite_text is set for post-processing with roche.C 
      logical scatter, write_text
      parameter (sigma = 10.d0, 
     &     pi = 3.14159265358979323846, au2pc = pi/(180*60*60),
     &     g = 6.67d-8*1.989d33*1.d-10/3.085678d18,as = 21.d0,
     &     t2myr = 3.085678d13/(1d6*31 556 925.9747d0),
     &     au2rsun=214.9,pc2rsun=4.434e+7)
         p_exc(m1,m2,m3) = 3**(-1.d0/6.d0)*2**(8.d0/3.d0)*m3**(3.5d0)*
     &        (m2+m3)**(1.d0/6.d0)*(m1+m3)**(-2.5d0)*
     &        (m1+m2)**(-1.d0/3.d0)*(m1+m2+m3)**(-5.d0/6.d0)

      scatter_counter = 0
      evolve_counter = 0
cNote: enter a in Rsun
      read (5,*) m1, m2, aRsun, e, tend, n, alpha, m_min, m_max, idum
      if (idum<=0) then
         idum = time()
      endif
cThe selection of the encountering particle would have to be altered
cif alpha < 1:
      if (alpha.lt.1.d0) then
         write (6,*) 'Choose alpha > 1; stopping'
         stop
      endif
cConvert to pc
      a = aRsun/pc2rsun
cInitialize Binary
      in = 1
      id = 0
      id3 = 2
      call init_binaries(in, id, aRsun, e, m1, m2)
      print *, 'Initial binary:'
      call out_binary(id)
c
cNormalisation factor for power-law imf:
      fnorm = (1.d0 - alpha)/(m_max**(1.d0-alpha) - m_min**(1.d0-alpha))
cMean (average) mass at zero age [for determining encounter rate]
      ma = fnorm*(m_max**(2.d0-alpha) - m_min**(2.d0-alpha))/
     &     (2.d0 - alpha)
      write (6,*) '#ma = ',ma
cInitialise scattering run
      t = 0.d0
cOutcome initialized to "preservation"
      outcome = 0 
      a_factor = 1;
*******************************************************************************
cMain loop starts here: 
*******************************************************************************
c
 10      continue
cBinding energy
      eb = g*m1*m2/(2*a)
cCoefficient in encounter rate
      coeff = 5*sqrt(2.d0)*pi*as*g*a*sqrt(m1+m2)/(16*sqrt(3.d0*ma)*
     &     sigma)
cEstimate of maximum total encounter rate
      rate_max = coeff*sqrt((m1+m2+m_max)*m_max)*n
c
cEstimate timestep to give probability of encounter < 0.1 and
cless than evolution time step
c      dt = 0.1d0/rate_max
      dt = 1d0/rate_max
c      dt = 10d0/rate_max
      call get_bs_updatetime(id, updt)
      dt = min(dt, updt)
c      print *, '+++dt = ', dt
c
cSelect third star and compute probability of encounter
      m3 = m_min + ran2(idum)*(m_max - m_min)
      p = coeff*sqrt((m1+m2+m3)*m3)*n*dt*fnorm*m3**(-alpha)
      random = ran2(idum)
      if (p.lt.random) then
         scatter = .false.
         t = t + dt
      else
cChoose a time for the encounter
         scatter = .true.
         scatter_counter = scatter_counter + 1
         t = t + dt*ran2(idum)
c        Scatter is going to take place
c        But if binary changed dramatically this is not taken into account
c        in the scatter cross section
      endif
      tMyr = t*t2Myr
c      print *, '+++t = ', tMyr

c
cEvolve binary star
cIf the previous encounter was an exchange (outcome>0), update the binary 
ccomponents now. Otherwise morphbinary does nothing.
      call morph_binary(id, id3, a_factor, e, outcome)
      if(outcome.gt.0)then
         write (6,*) 'between the binary: (next binary update at t=', 
cAltered by DCH 22/7/4
c     &        tupdt, ')'
     &        updt, ')'
         call out_binary(id)
         write (6,*) 'and the single star:'
         call out_star(id3)
         write (6,*) 'mass = ', m3, ' radius = ', r3
         write (6, *) 'Post encounter parameters: a=', aRsun, 
     &        ', e=', e,
     &        ', Eb=', eb
         print *
      endif
      call ev_binary(id, tMyr)
      evolve_counter = evolve_counter + 1
      call binary_exists(id, iexist)
      if(iexist.eq.0) then
         write (6,*) 'Binary has been ionized or is merged'
         stop
      endif
      call get_sma(id, aRsun)
      call get_ecc(id, e)
      call get_loid_mass(id, m1)
      call get_hiid_mass(id, m2)
c     Idially we would like to change the encounter probability here 
c     (but, for simplicity we don't).
c     update semi-major axis after binary evolution      
      a = aRsun/pc2rsun
cEnd evolve the binary
c
cPerform scatter experiment
      if(scatter.eqv..true.)then

         id3 = id3+1

c     Evolve third star
         call init_stars(in, id3, m3)
         call ev_star(id3, tmyr)
         call get_mass(id3, m3)
         call get_radius(id3, r3)
c     Third star evolved
c     in fact, the previous encounter rate         
         write (6,*) '#max encounter rate/Myr ',rate_max/t2myr
         write (6,*) '#encounter rate/Myr for average mass ',
     &        coeff*sqrt((m1+m2+ma)*ma)*n/t2myr
cDetermine whether an exchange has occurred (2 cases to consider)
cBased on: HHMcM 1996ApJ...467..359H
         write (6,*) m1, m2, m3
         p_exc1 = p_exc(m1,m2,m3)
	 p_exc2 = p_exc(m2,m1,m3)
         if(p_exc1+p_exc2.gt.1)then
            print *, 'Warning: exchange probability too large (old p=', 
     &               p_exc1, p_exc2, ')'
            p_exc_tot = p_exc1+p_exc2
            p_exc1 = p_exc1/p_exc_tot
            p_exc2 = p_exc2/p_exc_tot
         endif
         exch = ran2(idum)
         m3old = m3
         if (exch.lt.p_exc1) then
            write (6,*) 'exchange lowest id star'
            write_text = .false.
            m3 = m1
            m1 = m3old
            outcome = 2 
cAssume a is unchanged (this needs improvement)
            eb = g*m1*m2/(2*a)
cPresumably updating of the binary evolution is needed here
            a_factor = 1
         else if (exch.lt.p_exc1+p_exc2) then
            write (6,*) 'exchange highest id star'
            write_text = .false.
            m3 = m2
            m2 = m3old
            outcome = 1 
cAssume a is unchanged (this needs improvement)
            eb = g*m1*m2/(2*a)
cPresumably updating of the binary evolution is needed here
            a_factor = 1
         else 
cFlyby:
            outcome = 0 
            write_text = .true.
cChoose Delta; x = arctan(sqrt(Delta)), Spitzer's cross section, Delta > 0, 
cf(x) = cos^6(x)
 20         continue
            random = ran2(idum)
            x = random*pi/2
            f = ran2(idum)
            if (f.gt.cos(x)**6) go to 20
            delta = tan(x)**2
c     Alter binary parameters and rate coefficient
            eb = eb*(1+delta)
            a_factor = 1/(1+delta)
            a = a * a_factor

         endif
         aRsun = a*pc2rsun
c
c        initiate eccentricity in the encounter
         e = sqrt(ran2(idum))
c
         call out_scatter(id, id3, tMyr, write_text);
         print *
         write (6,*) 'Strong encounter at time =', tMyr
      else
         outcome = 0 
         a_factor = 1
      endif
c
cCheck for end of run
      if (tMyr.lt.tend) then
         goto 10
      else 
         write (6,*) tMyr,a*pc2rsun,eb
      endif
c
      write (6,*) 'Program terminated normally'
      write (6,*) 'Final binary:'
      call out_binary(id)
      write_text = .false.;
      call out_scatter(id, id3, tMyr, write_text);
      write (6, *) 'scatter_counter = ', scatter_counter
      write (6, *) 'evolve_counter = ', evolve_counter
c
      stop
      end

